#include <map>
#include <string>

#include <X11/Xlib.h>

////////////////////////////////////////////////////////////////////////////////
class App
   {
   public:

   /////////////////////////////////////
   class BitMap
      {
      public:
      BitMap(const std::string & file);
      ~BitMap();

      void make_pixmap(Display * dpy, Drawable d);

      // Yeah, it's all public here.  Keeping it simple.
      std::string name;
      unsigned int width;
      unsigned int height;
      unsigned char * data;
      Pixmap pixmap;

      private:
      BitMap(const App::BitMap & b);

      int read_from_file(const std::string & file);
      };

   /////////////////////////////////////
   typedef struct
      {
      public:
      std::string name;
      Window win;
      App::BitMap * icon;
      } WindowData;

   /////////////////////////////////////

   typedef std::pair<std::string, App::WindowData *> win_pair;
   typedef std::map<std::string, App::WindowData *> WindowMap;

   App(const std::string & appname);
   ~App();

   static int x_error_handler(Display * dpy, XErrorEvent *err);

   void create_window(const std::string& win_name,
		      int x, int y,
		      unsigned int width, unsigned int height,
		      App::BitMap * icon);

   inline int display_height(void) const
      { return DisplayHeight(dpy, DefaultScreen(dpy)); };

   inline int display_width(void) const
      { return DisplayWidth(dpy, DefaultScreen(dpy)); };

   private:
   App();
   App(const App & a);

   Display * dpy;
   WindowMap windows;
   };
